/*
 * Copyright (c) 2003 by muppet
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307  USA.
 */
#include "zvttermperl.h"
#include <gperl_marshal.h>

static SV *
newSVVTTITLE_TYPE (VTTITLE_TYPE type)
{
	switch (type) {
	    case VTTITLE_WINDOWICON: return newSVpv ("windowicon", 10); break;
	    case VTTITLE_ICON:       return newSVpv ("icon",        8); break;
	    case VTTITLE_WINDOW:     return newSVpv ("window",      6); break;
	    case VTTITLE_XPROPERTY:  return newSVpv ("xproperty",   9); break;
	    default:
		warn ("unexpected VTTITLE_TYPE value %d", type);
		return newSViv (type);
	}
}

/*
ZvtTerm's title-changed signal is defined incorrectly; the VTTITLE_TYPE
and const char * arguments are declared as int and pointer.  so, we'll
need to marshal these by hand.
*/

static void
zvttermperl_title_changed_marshal (GClosure * closure,
                                   GValue * return_value,
                                   guint n_param_values,
                                   const GValue * param_values,
                                   gpointer invocation_hint,
                                   gpointer marshal_data)
{
	int len;
	gint * position_p;
	SV * string, * position;
	dGPERL_CLOSURE_MARSHAL_ARGS;

	GPERL_CLOSURE_MARSHAL_INIT (closure, marshal_data);

	PERL_UNUSED_VAR (return_value);
	PERL_UNUSED_VAR (n_param_values);
	PERL_UNUSED_VAR (invocation_hint);

	ENTER;
	SAVETMPS;

	PUSHMARK (SP);

	GPERL_CLOSURE_MARSHAL_PUSH_INSTANCE (param_values);

	/* VTTITLE_TYPE type, as a G_TYPE_INT */
	XPUSHs (sv_2mortal (newSVVTTITLE_TYPE (g_value_get_int (param_values+1))));
	/* const char * title, as an infamous G_TYPE_POINTER */
	XPUSHs (sv_2mortal (newSVpv (g_value_get_pointer (param_values+2), 0)));

	GPERL_CLOSURE_MARSHAL_PUSH_DATA;

	PUTBACK;

	GPERL_CLOSURE_MARSHAL_CALL (G_VOID|G_DISCARD);

	FREETMPS;
	LEAVE;
}

#include "const-c.inc"

MODULE = Gnome2::ZvtTerm PACKAGE = Gnome2::ZvtTerm PREFIX = zvt_term_

INCLUDE: const-xs.inc

BOOT:
	gperl_register_object (ZVT_TYPE_TERM, "Gnome2::ZvtTerm");
	gperl_signal_set_marshaller_for (ZVT_TYPE_TERM, "title_changed",
	                                 zvttermperl_title_changed_marshal);


GtkWidget * zvt_term_new (SV * class);
    C_ARGS:
	/*void*/

GtkWidget * zvt_term_new_with_size (SV * class, int cols, int rows);
    C_ARGS:
	cols, rows

GtkAdjustment*
adjustment(term)
	ZvtTerm	* term
    CODE:
	RETVAL = term->adjustment;
    OUTPUT:
	RETVAL

void zvt_term_reset (ZvtTerm *term, gboolean hard);

void zvt_term_feed (ZvtTerm *term, gchar_length *text, int length(text));

int zvt_term_writechild (ZvtTerm *term, gchar_length *data, int length(data));

=for apidoc
=for arg do_uwtmp_log (flags) see the ZVT_TERM_DO_* constants.
=cut
int zvt_term_forkpty (ZvtTerm *term, int do_uwtmp_log);

int zvt_term_closepty (ZvtTerm *term);

int zvt_term_killchild (ZvtTerm *term, int signal);

void zvt_term_bell (void *zvt_term);

void zvt_term_set_scrollback (ZvtTerm *term, int lines);

##char * zvt_term_get_buffer (ZvtTerm *term, int *len, int type, int sx, int sy, int ex, int ey);
void
zvt_term_get_buffer (term, type, sx, sy, ex, ey)
	ZvtTerm* term
	int	type
	int	sx
	int	sy
	int	ex
	int	ey
    PREINIT:
	char* res;
	int len=0;
    PPCODE:
	res = zvt_term_get_buffer (term, &len, type, sx, sy, ex, ey);
	EXTEND (sp, 2);
	PUSHs (sv_2mortal (newSVpv (res, 0)));
	PUSHs (sv_2mortal (newSViv (len)));
	g_free (res);


#ifdef HAS_I18N

##gchar* zvt_term_locale_from_utf8 (const gchar *utf8_ptr, const gssize utf8_len, gsize *bytes_read, gsize *bytes_written, GError **error);

##gchar* zvt_term_locale_to_utf8 (const gchar *locale_ptr, const gssize locale_len, gsize *bytes_read, gsize *bytes_written, GError **error);

 ##void zvt_term_set_pango_font (ZvtTerm *term, const PangoFontDescription*);
void zvt_term_set_pango_font (ZvtTerm *term, const PangoFontDescription* font_desc);

##gboolean zvt_localechar_iswide (gunichar);
##gint zvt_column_from_utf8 (gchar *, gint);
##
##void zvt_term_im_new (ZvtTerm *term);
##void zvt_term_im_cursor_location (ZvtTerm *term);
##void zvt_term_preedit_update (ZvtTerm *term);
##void zvt_term_im_reset (ZvtTerm *term);
##void zvt_term_im_free (ZvtTerm *term);
##
##void zvt_term_get_baseline_offset (ZvtTerm *term, int row, int col, gint *offset_x, gint *offset_y);
##void zvt_term_draw_text (ZvtTerm *term, struct vt_line *line, int row, int col, gchar *utf8_ptr, int len, int x, int y, int offx, int offy, GdkGC *fgc, int dofill, int overstrike);
##
##gchar* zvt_term_get_utf8_from_line (struct vt_line* line);
##gint zvt_term_get_char_index_from_column (gchar* utf8_ptr, gint column_index); 
##gint zvt_term_get_column_from_char_index (gchar* utf8_ptr, gint char_index);

#endif

##void zvt_term_scroll (ZvtTerm *term, int n);

void zvt_term_set_font_name (ZvtTerm *term, char *name);

 ## GdkFonts are deprecated in Gtk2, so we will probably never bind this.
###void zvt_term_set_fonts (ZvtTerm *term, GdkFont *font, GdkFont *font_bold);

void zvt_term_hide_pointer (ZvtTerm *term);

void zvt_term_show_pointer (ZvtTerm *term);

void zvt_term_set_bell (ZvtTerm *term, int state);

gboolean zvt_term_get_bell (ZvtTerm *term);

void zvt_term_set_blink (ZvtTerm *term, int state);

void zvt_term_set_scroll_on_keystroke (ZvtTerm *term, int state);

void zvt_term_set_scroll_on_output (ZvtTerm *term, int state);

#void zvt_term_set_color_scheme (ZvtTerm *term, gushort *red, gushort *grn, gushort *blu);
void
zvt_term_set_color_scheme (term, red, green, blue)
	ZvtTerm * term
	SV * red
	SV * green
	SV * blue
    PREINIT:
	gushort r[18], g[18], b[18];
	AV *ra, *ga, *ba;
	int i;
	SV **s;
    CODE:
	if (!red || !SvOK (red) || !SvROK (red) ||
	    SvTYPE (SvRV(red)) != SVt_PVAV)
		croak ("need an array ref in set_color_scheme");
	ra = (AV*) SvRV (red);
	if (!green || !SvOK (green) || !SvROK (green) ||
	    SvTYPE (SvRV (green)) != SVt_PVAV)
		croak ("need an array ref in set_color_scheme");
	ga = (AV*) SvRV (green);
	if (!blue || !SvOK (blue) || !SvROK (blue) ||
	    SvTYPE (SvRV (blue)) != SVt_PVAV)
		croak ("need an array ref in set_color_scheme");
	ba = (AV*) SvRV (blue);
	for (i=0; i < 18; ++i) {
		r[i] = (s=av_fetch (ra, i, 0)) && SvOK (*s) ? SvIV (*s) : 0;
		g[i] = (s=av_fetch (ga, i, 0)) && SvOK (*s) ? SvIV (*s) : 0;
		b[i] = (s=av_fetch (ba, i, 0)) && SvOK (*s) ? SvIV (*s) : 0;
	}
	zvt_term_set_color_scheme (term, r, g, b);


void zvt_term_set_default_color_scheme (ZvtTerm *term);

void zvt_term_set_del_key_swap (ZvtTerm *term, int state);

#void zvt_term_set_del_is_del (ZvtTerm *term, int state);
#void zvt_term_set_wordclass (ZvtTerm *term, unsigned char *klass);

void zvt_term_set_auto_window_hint (ZvtTerm *term, int state);

#void zvt_term_set_backspace_binding (ZvtTerm *term, ZvtEraseBinding binding);
#void zvt_term_set_delete_binding (ZvtTerm *term, ZvtEraseBinding binding);


## regular expression matching - automagically!

int zvt_term_match_add (ZvtTerm *term, char *regex, guint32 highlight_mask, void *data);

void zvt_term_match_clear (ZvtTerm *term, char *regex);

##char * zvt_term_match_check (ZvtTerm *term, int x, int y, void **user_data_ptr);


##
## transparency stuff, it's left in even if we don't compile
## transparency/backround pixmaps, if we don't, it will just be ignored,
## setting pixmap_file to NULL disables the background pixmap
##

=for apidoc

=cut
void zvt_term_set_background (ZvtTerm *terminal, char *pixmap_file, int transparent, int flags);

void zvt_term_set_background_with_shading(ZvtTerm *terminal, char *pixmap_file, int transparent, int flags, guint16 red, guint16 green, guint16 blue, guint16 alpha);

void zvt_term_set_shadow_type (ZvtTerm *term, GtkShadowType  type);

void zvt_term_set_size (ZvtTerm *term, guint width, guint height);


## returns an bitmask of the capabilities compiled into ZvtTerm

guint32 zvt_term_get_capabilities (ZvtTerm *term);


## clipboard operations for menu use

void zvt_term_copy_clipboard (ZvtTerm *term);

void zvt_term_paste_clipboard (ZvtTerm *term);
